%EFILTER2 Two-dimensional filter with bounding.
%   Y = EFILTER2(KER,X) filters the image in X with the 2-D FIR
%   filter in the matrix KER.  The result, Y, is computed 
%   using 2-D correlation and is the same size as X. 
%   Before applying the filter, image X is exapnded by the half
%   of KER size using linear interpolation, in order to avoid 
%   border effects inside the image.

% Zvi Devir & Liliya Tridensky
% Sep 2002

function B = eFilter2(kernel, A);

n = floor(max(size(kernel))/2);

% Expand the image by the half the kernel size, in order to 
% filter border pixels correctly.
B = expandImage(A,n);

B = filter2(kernel, B);

% Reduce to the original size
B = B(n+1:end-n, n+1:end-n); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%EXPANGIMAGE Expand image using linear interpolation.
%   Y = EXPANDIMAGE(X, N) expands the image in X by N pixels
%   in each direction.
%   Y = EXPANDIMAGE(X) expands the image by one pixel.

function B = expandImage(A, n);

MEAN3 = [1 1 1]/3;

if (nargin<2), n=1; end;
n = max(n);

B = A;

for rep=1:n,
  A = B;
  [sy sx] = size(A);
  B = zeros([sy+2, sx+2]);
  Y1 = A(1,:);
  Y2 = A(sy,:);
  X1 = A(:,1);
  X2 = A(:,sx);
    
  Y1 = [Y1(1), Y1, Y1(sx)];
  Y2 = [Y2(1), Y2, Y2(sx)];
  X1 = [X1(1); X1; X1(sy)];
  X2 = [X2(1); X2; X2(sy)];
    
  Y1 = conv(MEAN3, Y1);
  Y2 = conv(MEAN3, Y2);
  X1 = conv(MEAN3, X1);
  X2 = conv(MEAN3, X2);
    
  B(2:sy+1,2:sx+1) = A;
  B(1,:) = Y1(2:sx+3);
  B(sy+2,:) = Y2(2:sx+3);
  B(:,1) = X1(2:sy+3);
  B(:,sx+2) = X2(2:sy+3);
    
  B(1,1) = (Y1(2)+X1(2)) * 0.75;
  B(1,sx+2) = (Y1(sx+3)+X2(2)) * 0.75;
  B(sy+2,1) = (Y2(2)+X1(sy+3)) * 0.75;
  B(sy+2,sx+2) = (Y2(sx+3)+X2(sy+3)) * 0.75;
end;
