% RETINEX Retinex Algorithm
%    L = RETINEX(X, alpha, beta) runs the Retinex algorithm over
%    the image in X, with the parameters alpha and beta. X is a 
%    grayscale image.
%    L is an approximation of the light component in the image.

% Reference: 
%    R.Kimmel, D.Shaked, M.Elad - "A Variational Framework for Retinex".

% Zvi Devir & Liliya Tridensky
% Sep 2002

function L = retinex(A, alpha, beta);

P = 4; % Number of algorithm steps
% Number of iteration in each step:
%T = [2,4,8,16];
T = [4,4,8,16];
%T = [8,8,12,16];

h = waitbar(0, 'Calculating RETINEX');
waitbar(0);

Kpyr = [1/16, 1/8, 1/16; 1/8, 1/4, 1/8; 1/16, 1/8, 1/16]; % Gaussian kernel
Klap = [0, 1, 0; 1, -4, 1; 0, 1, 0]; % Laplacian kernel

% Create Gaussian pyramid:
S = cell([1,P]);
S{1} = A;
for i=2:P,
  A = expand(A); % Expanding the image for even number of rows and colomns
  A = eFilter2(Kpyr, A); % Smoothing the image
  A = A(1:2:end,1:2:end) + A(2:2:end,1:2:end) + A(1:2:end,2:2:end) + A(2:2:end,2:2:end);
  A = A .* 0.25;
  S{i} = A;
% figure; imshow(A);
end;


Sk = S{P};
L = Sk;
for k=P:-1:1,
  Klapk = Klap*2^(-2*(k-1));
% Klapk = Klap;
  Sk = S{k};
  Gb = eFilter2(Klapk, Sk);
  
  for j=1:T(k),
    Ga = eFilter2(Klapk, L);
    G = -Ga + alpha*(L-Sk) - beta*(Ga-Gb);
    ma = sum(G(:).*G(:));
    dG = eFilter2(Klapk, G);
    mb = -sum(G(:).*dG(:));
    m = ma / (alpha*ma + (1+beta)*mb);
    L = L - m*G;
    L = max(L,Sk);
    waitbar((P-k+j/T(k))/P);
  end;
  
  if k>1,
     L = kron(L, ones(2)); % Expand L to the next level
     [x y] = size(S{k-1});
     L = L(1:x,1:y);
  end;
  
end;

close(h);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This function expands the image in order to get an even 
% number of rows and colomns for gaussian filtering

function B = expand(A);

[x y] = size(A);

if (rem(x,2)~=0)
      B = [A; A([x],:)];
   if (rem(y,2)~=0)
      B = [B B(:,[y])];
   end
elseif (rem(y,2)~=0)
      B = [A A(:,[y])];
else
   B = A;
end
