% RETINEX2 Remove light component from image
%    R = RETINEX2(S,a,b) uses the Retinex algorithm to approximate
%    the reflectance image from the sourse image S, by removing 
%    the illumination image. S is a grayscale image. 
%    R = RETINEX2(S,a,b,c) uses gamma correction. 
%    R = RETINEX(S) uses default values for alpha, beta and gamma.

% Reference: 
%    R.Kimmel, D.Shaked, M.Elad - "A Variational Framework for Retinex".

% Zvi Devir & Liliya Tridensky
% Sep 2002


function S = retinex2(S, alpha, beta, gamma);

% Deafult retinex parameters
if (nargin<4), gamma = inf; end; 
if (nargin<3), beta = 0.1; end;
if (nargin<2), alpha = 0.0001; end;

S = log(S*0.95+0.05); % in order to avoid log by zero
L = retinex(S, alpha, beta);
R = S-L;
L = L./gamma; % gamma correction 
S = L+R;
S = exp(S);
