%STRETCHIMAGE Stretch intensity image with minimum of saturated pixels.
%   B = STRETCHIMAGE(A,S) stretches the intensity image A to the range
%   of [0,1] in order to increase the image visual range, but saturates 
%   no more than S*size(A) pixels. 
%   By throwing away rared used colors in the edges of the intensity space, 
%   we get more enhanced image.
%   
%   B = STRETCHIMAGE(A,0) just stretches the intensity image A to the range
%   of [0,1] without any saturation at all.
%   
%   B = STRETCHIMAGE(A) saturates 0.5% of the pixels.

% Zvi Devir & Liliya Tridensky
% Sep 2002

function B = stretchImage(A,s);

if nargin<2, s = 0.005; end;    % Default - loose 0.5%
s = prod(size(A))*s;

if s==0, % Just stretch the image to [0,1]
    l = min(A(:));
    h = max(A(:));
else
    [H1,C] = fullHist(A);
    H2 = flipud(prod(size(A))-H1);
    H1 = [0;H1];
    n = 0;
    l = 1; h = 1;

    while n<s,
      if H1(l+1)<=H2(h+1),
        l = l+1;
      else
        h = h+1;
      end;
      n = H1(l)+H2(h);
    end;

    l = C(l);
    h = C(end-h+1);
    clear s n C H1 H2
end;

disp(sprintf('\tRange: %f - %f',l,h));
B = (A-l)./(h-l);
B = max(min(B,1),0);